﻿using Mapbox.Unity;
using System;
using UnityEngine;
using UnityEngine.UI;
using Mapbox.Json;
using Mapbox.Directions;
using Mapbox.Utils;
using Mapbox.Utils.JsonConverters;
using Mapbox.Geocoding;

public class destination : MonoBehaviour
{
	[SerializeField]
	Text _resultsText;

	[SerializeField]
	ForwardGeocodeUserInput _startLocationGeocoder;

	[SerializeField]
	ForwardGeocodeUserInput _endLocationGeocoder;

	Directions _directions;

	Vector2d[] _coordinates;

	DirectionResource _directionResource;

	void Start()
	{
		_directions = MapboxAccess.Instance.Directions;
        _startLocationGeocoder.OnGeocoderResponse += _startLocationGeocoder_OnGeocoderResponse;
        _endLocationGeocoder.OnGeocoderResponse += _endLocationGeocoder_OnGeocoderResponse;

		_coordinates = new Vector2d[2];

		// Can we make routing profiles an enum?
		_directionResource = new DirectionResource(_coordinates, RoutingProfile.Driving);
		_directionResource.Steps = true;
	}

    private void _endLocationGeocoder_OnGeocoderResponse(ForwardGeocodeResponse obj)
    {
		_coordinates[1] = _endLocationGeocoder.Coordinate;
		if (ShouldRoute())
		{
			Route();
		}
	}

    private void _startLocationGeocoder_OnGeocoderResponse(ForwardGeocodeResponse obj)
    {
		_coordinates[0] = _startLocationGeocoder.Coordinate;
		if (ShouldRoute())
		{
			Route();
		}
	}

	/// <summary>
	/// Ensure both forward geocoders have a response, which grants access to their respective coordinates.
	/// </summary>
	/// <returns><c>true</c>, if both forward geocoders have a response, <c>false</c> otherwise.</returns>
	bool ShouldRoute()
	{
		return _startLocationGeocoder.HasResponse && _endLocationGeocoder.HasResponse;
	}

	/// <summary>
	/// Route 
	/// </summary>
	void Route()
	{
		_directionResource.Coordinates = _coordinates;
		_directions.Query(_directionResource, HandleDirectionsResponse);
	}
	/// <summary>
	/// Log directions response to UI.
	/// </summary>
	/// <param name="res">Res.</param>
	void HandleDirectionsResponse(DirectionsResponse res)
	{
		var data = JsonConvert.SerializeObject(res, Formatting.Indented, JsonConverters.Converters);
		string sub = data.Substring(0, data.Length > 5000 ? 5000 : data.Length) + "\n. . . ";
		_resultsText.text = sub;
		System.IO.File.WriteAllText(Application.dataPath + "\\test.json", data);
	}


}
